/* SPDX-License-Identifier: (GPL-2.0+ or MIT) */
/*
 * Copyright (c) 2019 Martin Blumenstingl <martin.blumenstingl@googlemail.com>
 */

#ifndef _DT_BINDINGS_MESON8_POWER_H
#define _DT_BINDINGS_MESON8_POWER_H

#define PWRC_MESON8_VPU_ID			0
#define PWRC_MESON8_ETHERNET_MEM_ID		1
#define PWRC_MESON8_AUDIO_DSP_MEM_ID		2

#endif /* _DT_BINDINGS_MESON8_POWER_H */
