/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2019-2021 Linaro Ltd.
 *
 * Author:
 * Sumit Garg <sumit.garg@linaro.org>
 */

#ifndef __TEE_TRUSTED_KEY_H
#define __TEE_TRUSTED_KEY_H

#include <keys/trusted-type.h>

extern struct trusted_key_ops trusted_key_tee_ops;

#endif
