/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides constants for gpio keys bindings.
 */

#ifndef _DT_BINDINGS_GPIO_KEYS_H
#define _DT_BINDINGS_GPIO_KEYS_H

#define EV_ACT_ANY		0x00	/* asserted or deasserted */
#define EV_ACT_ASSERTED		0x01	/* asserted */
#define EV_ACT_DEASSERTED	0x02	/* deasserted */

#endif /* _DT_BINDINGS_GPIO_KEYS_H */
