/* SPDX-License-Identifier: GPL-2.0-or-later */
/* Keyring key type
 *
 * Copyright (C) 2008, 2013 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 */

#ifndef _KEYS_KEYRING_TYPE_H
#define _KEYS_KEYRING_TYPE_H

#include <linux/key.h>
#include <linux/assoc_array.h>

#endif /* _KEYS_KEYRING_TYPE_H */
